[tool.poetry]
name = "{{project_name}}"
version = "{{version}}"
description = "Autogenerated httpx async client for {{project_name}}"

[tool.poetry.dependencies]
python = "^3.7"
httpx = ">=0.22, <1"
pydantic = "^1"

[tool.poetry.scripts]
poetry = "poetry.console:main"

[build-system]
requires = ["poetry-core>=1.0.0"]
build-backend = "poetry.core.masonry.api"
